@@ -1,13 +1,14 @@
 """Gateway connecting to ATAG thermostat."""
-import aiohttp
-import logging
-from datetime import datetime, timedelta
-import re
-import uuid
 import asyncio
 import logging
-from .errors import raise_error
-from .entities import Report, Climate, DHW
+import re
+import uuid
+from datetime import datetime, timedelta
+
+import aiohttp
+from pyatag import errors
+
+from .entities import DHW, Climate, Report
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -48,6 +49,9 @@ class AtagOne:
 
     async def authorize(self):
         """Check auth status."""
+        if self._authorized:
+            _LOGGER.debug("Not checking auth status as ID was provided")
+            return True
         json = {
             "pair_message": {
                 "seqnr": 1,
@@ -64,52 +68,79 @@ class AtagOne:
                 },
             }
         }
-        await self.request("post", "pair", json)
-        self.authorized = True
-        _LOGGER.debug(f"Authorized: {self.authorized}")
-        return self.authorized
+        try:
+            await self.request("post", "pair", json)
 
-    async def request(self, meth, path, json=None):
+        except errors.Unauthorized as err:
+            _LOGGER.debug("Received unauthorized message, try with email address")
+            raise err
+        except errors.AtagException as err:
+            _LOGGER.debug("Authorization failed: %s", type(err))
+            raise err
+        self._authorized = True
+        _LOGGER.debug("Authorized successfully")
+        return True
+
+    async def request(self, meth, path, json=None, force=False):
         """Make a request to the API."""
         url = f"http://{self.host}:{self.port}/{path}"
         async with self._lock:
             slept = (datetime.utcnow() - self._last_call).total_seconds()
-            if slept < 5:
-                await asyncio.sleep(5 - slept)
+            if slept < 3 and not force:
+                _LOGGER.debug("Sleeping for %ss", 3 - slept)
+                await asyncio.sleep(3 - slept)
             self._last_call = datetime.utcnow()
-            try:
-                async with self.session.request(meth, url, json=json) as res:
-                    data = await res.json()
-                    _raise_on_error(data)
-                    return data
-            except aiohttp.ClientConnectorError as err:
-                raise_error(err, 2)
-            except Exception as err:
-                raise_error(err, 5)
+            for tries in range(4):
+                try:
+                    _LOGGER.debug("Calling %s for %s", self.host, path)
+                    async with self.session.post(url, json=json) as res:
+                        data = await res.json()
+                        _raise_on_error(data)
+                        return data
+                except aiohttp.ServerDisconnectedError as err:
+                    if tries == 3:
+                        _LOGGER.debug(
+                            "Server disconnected unexpectedly 3 times, giving up"
+                        )
+                        errors.raise_error(err, 2)
+                    _LOGGER.debug("Server disconnected unexpectedl: %s", tries)
+                except aiohttp.ClientConnectionError as err:
+                    _LOGGER.debug("Failed to connect to %s", self.host)
+                    errors.raise_error(err, 2)
+                except aiohttp.ClientResponseError as err:
+                    _LOGGER.debug("Caught response error: %s", err)
+                    errors.raise_error(err, 3)
+                except aiohttp.InvalidURL as err:
+                    _LOGGER.debug("Could not connect, url %s is incorrect", url)
+                    errors.raise_error(err, 2)
+                except (aiohttp.ClientError, asyncio.TimeoutError) as err:
+                    _LOGGER.debug("Unknown error occurred")
+                    errors.raise_error(err, 5)
 
     async def update(self, info=71, force=False):
         """Get latest data from API."""
-        if (datetime.utcnow() - self._last_call).total_seconds() > 15 or force:
-            json = {
-                "retrieve_message": {
-                    "seqnr": 1,
-                    "account_auth": {
-                        "user_account": self.email,
-                        "mac_address": self._mac,
-                    },
-                    "info": info,
-                }
+        json = {
+            "retrieve_message": {
+                "seqnr": 1,
+                "account_auth": {"user_account": self.email, "mac_address": self._mac},
+                "info": info,
             }
-            res = await self.request("get", "retrieve", json)
-            res = res["retrieve_reply"]
-            res["report"].update(res["report"].pop("details"))
-            if self.report is None:
-                self.report = Report(res, self.update, self.setter)
-                self.climate = Climate(self.report)
-                self.dhw = DHW(self.report)
+        }
+        try:
+            _LOGGER.debug("Queueing data update")
+            res = await self.request("post", "retrieve", json, force)
+        except errors.AtagException as err:
+            _LOGGER.debug("Update failed: %s", err)
+            raise err
+        res = res["retrieve_reply"]
+        res["report"].update(res["report"].pop("details"))
+        if self.report is None:
+            self.report = Report(res, self.update, self.setter)
+            self.climate = Climate(self.report)
+            self.dhw = DHW(self.report)
+        else:
             self.report.update(res)
-            return self.report
-        return None
+        return True
 
     async def setter(self, **kwargs):
         """Set control items."""
@@ -131,17 +162,26 @@ class AtagOne:
                 json["update_message"]["configuration"]["start_vacation"] = int(
                     (datetime.utcnow() - datetime(2000, 1, 1)).total_seconds()
                 )
-        res = await self.request("post", "update", json)
+        try:
+            res = await self.request("post", "update", json)
+        except errors.Unauthorized:
+            raise errors.Unauthorized(
+                "Failed to set Atag, received unauthorized message"
+            )
+        except errors.RequestError:
+            raise errors.RequestError("Failed to set Atag, could not complete request")
+        except errors.AtagException:
+            raise errors.UnknownAtagError("Failed to set Atag, unknown error occurred")
         return res["update_reply"]
 
 
 def _raise_on_error(data):
     """Check response for error message."""
     if data[list(data.keys())[0]]["acc_status"] != 2:
-        raise_error(data, 1)
+        errors.raise_error(data, 1)
 
     if isinstance(data, list):
         data = data[0]
 
     if isinstance(data, dict) and "error" in data:
-        raise_error(data)
+        errors.raise_error(data)
